<?php

namespace ByDedi\AdminBundle\Admin;

use ByDedi\AdminBundle\Form\Type\SecurityRolesType;
use Doctrine\ORM\QueryBuilder;
use Sonata\AdminBundle\Datagrid\DatagridMapper;
use Sonata\AdminBundle\Datagrid\ListMapper;
use Sonata\AdminBundle\Form\FormMapper;
use Sonata\AdminBundle\Route\RouteCollection;
use Sonata\AdminBundle\Show\ShowMapper;
use Sonata\UserBundle\Model\UserManagerInterface;

class UserAdmin extends BaseAdmin
{

    /**
     * @var UserManagerInterface
     */
    protected $userManager;


    /**
     * {@inheritdoc}
     */
    public function getFormBuilder()
    {
        $this->formOptions['data_class'] = $this->getClass();

        $options = $this->formOptions;
        $options['validation_groups'] = (!$this->getSubject() || is_null($this->getSubject()->getId())) ? 'Registration' : 'Profile';

        $formBuilder = $this->getFormContractor()->getFormBuilder($this->getUniqid(), $options);

        $this->defineFormBuilder($formBuilder);

        return $formBuilder;
    }

    /**
     * {@inheritdoc}
     */
    public function getExportFields()
    {
        // avoid security field to be exported
        return array_filter(parent::getExportFields(), function ($v) {
            return !in_array($v, array('password', 'salt'));
        });
    }

    /**
     * {@inheritdoc}
     */
    public function preUpdate($user)
    {
        $this->getUserManager()->updateCanonicalFields($user);
        $this->getUserManager()->updatePassword($user);
        //        dump($user);exit;
    }

    /**
     * @param UserManagerInterface $userManager
     */
    public function setUserManager(UserManagerInterface $userManager)
    {
        $this->userManager = $userManager;
    }

    /**
     * @return UserManagerInterface
     */
    public function getUserManager()
    {
        return $this->userManager;
    }

    /**
     * {@inheritdoc}
     */
    protected function configureListFields(ListMapper $listMapper)
    {
        $listMapper
            ->addIdentifier('id', null, ['label' => 'ID'])
            ->addIdentifier('username', null, ['label' => 'Nom'])
            ->add('email', null)
            ->add('groups', null, [
                'label' => 'Groupes'
            ])
            ->add('enabled', null, array('editable' => true, 'label' => 'Actif'))
//            ->add('locked', null, array('editable' => true, 'label' => 'Bloqué'))
            ->add('createdAt', null, [
                'label' => 'Date de création'
            ]);

        if ($this->isGranted('ROLE_ALLOWED_TO_SWITCH')) {
            $listMapper
                ->add('impersonating', 'string', array('template' => 'AdminBundle:admin/CRUD:list__impersonating.html.twig'));
        }

        $listMapper
            ->add('_action', 'actions', $this->getDefaultListActionParams());
    }

    /**
     * {@inheritdoc}
     */
    protected function configureDatagridFilters(DatagridMapper $filterMapper)
    {
        $filterMapper
            ->add('id')
            ->add('username')
//            ->add('locked')
            ->add('email')
            ->add('groups')
            ->add('enabled')
            ->add('createdAt', 'doctrine_orm_callback', array(
                'field_type' => 'text',
                'callback' => [$this, 'getDateFilterCallback']
            ), null, ['attr' => ['class' => 'datepicker']]);
        ;
    }

    /**
     * {@inheritdoc}
     */
    protected function configureShowFields(ShowMapper $showMapper)
    {
        $showMapper
            ->with('Identifiants')
            ->add('username')
            ->add('email')
            ->end()
            ->with('Groupes')
            ->add('groups')
            ->end()
            /*->with('Profil')
            ->add('dateOfBirth')
            ->add('firstname')
            ->add('lastname')*/
//            ->add('website')
//            ->add('biography')
//            ->add('gender')
//            ->add('locale')
//            ->add('timezone')
//            ->add('phone')
//            ->end()
            /*  ->with('Social')
              ->add('facebookUid')
              ->add('facebookName')
              ->add('twitterUid')
              ->add('twitterName')
              ->add('gplusUid')
              ->add('gplusName')
              ->end()*/
            ->with('Securité')
            ->add('token')
            ->add('twoStepVerificationCode')
            ->end();
    }

    /**
     * {@inheritdoc}
     */
    protected function configureFormFields(FormMapper $formMapper)
    {
        // define group zoning
        $formMapper
            ->tab('Utilisateur')
//            ->with('Profil', array('class' => 'col-md-6'))->end()
            ->with('Identifiants', array('class' => 'col-md-12'))->end()
//            ->with('Social', array('class' => 'col-md-6'))->end()
            ->end()
            ->tab('Securité')
            ->with('Statut', array('class' => 'col-md-6'))->end()
            ->with('Groupes', array('class' => 'col-md-6'))->end()
//            ->with('Keys', array('class' => 'col-md-4'))->end()
//            ->with('Roles', array('class' => 'col-md-12'))->end()
            ->end();

        $now = new \DateTime();

        $formMapper
            ->tab('Utilisateur')
            ->with('Identifiants')
            ->add('username', null, ['label' => 'Nom'])
            ->add('email')
            ->add('plainPassword', 'text', [
                'label' => 'Mot de passe',
                'required' => (!$this->getSubject() || is_null($this->getSubject()->getId())),
            ])
            ->end()
//            ->with('Profil')
            /*->add('dateOfBirth', 'sonata_type_date_picker', array(
                'years' => range(1900, $now->format('Y')),
                'dp_min_date' => '1-1-1900',
                'dp_max_date' => $now->format('c'),
                'required' => false,
            ))
            ->add('firstname', null, array('required' => false))
            ->add('lastname', null, array('required' => false))*/
//            ->add('website', 'url', array('required' => false))
//            ->add('biography', 'text', array('required' => false))
            /*->add('gender', 'sonata_user_gender', array(
                'required' => true,
                'translation_domain' => $this->getTranslationDomain(),
            ))*/
//            ->add('locale', 'locale', array('required' => false))
//            ->add('timezone', 'timezone', array('required' => false))
//            ->add('phone', null, array('required' => false))
//            ->end()
            /*   ->with('Social')
               ->add('facebookUid', null, array('required' => false))
               ->add('facebookName', null, array('required' => false))
               ->add('twitterUid', null, array('required' => false))
               ->add('twitterName', null, array('required' => false))
               ->add('gplusUid', null, array('required' => false))
               ->add('gplusName', null, array('required' => false))
               ->end()*/
            ->end()
            ->tab('Securité')
            ->with('Statut')
//            ->add('locked', null, array('required' => false))
//            ->add('expired', null, array('required' => false))
            ->add('enabled', null, [
                'label' => 'Disponible',
                'required' => false,
            ])
//            ->add('credentialsExpired', null, array('required' => false))
            ->end()
            ->with('Groupes')
            ->add('groups', 'sonata_type_model', [
                'label' => 'Groupes',
                'required' => false,
                'expanded' => true,
                'multiple' => true,
            ])
            ->end()
            /* ->with('Roles')
             ->add('realRoles', SecurityRolesType::class, array(
                 'label' => 'form.label_roles',
                 'expanded' => true,
                 'multiple' => true,
                 'required' => false,
             ))
             ->end()*/
            /* ->with('Keys')
             ->add('token', null, array('required' => false))
             ->add('twoStepVerificationCode', null, array('required' => false))
             ->end()*/
            ->end();
    }


    protected function configureRoutes(RouteCollection $collection)
    {
        parent::configureRoutes($collection);
        $collection->remove('clone');
    }
}
